void clearLEDs()
{
  for (byte i = 0; i < 5; i++) // if there's only one line in the for-loop, we don't have to use curly braces {}
    LEDrow[i] = 0; // 0 in this case is the same as B00000000, ie. all bits/LEDs off
}

void updateLEDs()
{
  // we can't shift out all 5 bytes at once
  // if we were to send all 5 at once and all 40 of the LEDs were on, we'd draw so much current that we'd starve the processor (trust me, I've done this)
  // instead, we alternate between sending the top 2 rows (and the bottom 3 off), and the bottom 3 rows (with the top 2 off)
  // this way at most 24 LEDs can be on at once
  // we'll need a variable to keep track of which rows we sent last
  static boolean lastSentTop = false; 

  digitalWrite(LEDlatchPin, LOW); // we ground the LEDlatchPin and hold it low for as long as we are shifting bytes out

  if (!lastSentTop) // send the top 2 rows - also keep in mind that we need to send the rows in reverse (sending the bottom row first and the top row last)
  {
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, B00000000); // row 5 for the sake of clarity, I'm not using a for-loop here
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, B00000000); // row 4
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, B00000000); // row 3
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, LEDrow[1]); // row 2 
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, LEDrow[0]); // row 1
    lastSentTop = true;
  }
  else
  {
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, LEDrow[4]);
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, LEDrow[3]); 
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, LEDrow[2]);
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, B00000000); 
    shiftOut(LEDdataPin, LEDclockPin, LSBFIRST, B00000000);
    lastSentTop = false;
  }
  digitalWrite(LEDlatchPin, HIGH); //we return the latch pin high to stop shifting
}

void assignLEDs()
{
  switch (mode)
  {
  case 0: // EDIT MODE
    if (showingNumbers)
    {
      showNumber();
    }
    else
    {
      for (byte i = 0; i < 4; i++)
      {
        switch(currentVoice)
        {
        case 0:
          LEDrow[i] = kick[i];
          break;
        case 1:
          LEDrow[i] = snare[i];
          break;
        case 2:
          LEDrow[i] = hat[i];
          break;
        case 3:
          LEDrow[i] = crash[i];
          break;
        }
      }

      if (seqRunning)
        LEDrow[currentStep / 8] = LEDrow[currentStep / 8] ^ 1 << (7 - (currentStep % 8)); // toggle the LED for the current step

      LEDrow[4] = bitSet(LEDrow[4], (6 - currentVoice)); // the control row
    }
    break;

  case 2: // TRIGGER MODE
    if (showingNumbers)
    {
      showNumber();
    }
    else
    {
      for (byte i = 0; i < 4; i++)
        LEDrow[i] = toc[trigPage*4 + i];

      // the control row
      LEDrow[4] = B10000000; 
      LEDrow[4] = bitSet(LEDrow[4], (6 - trigPage)); // which trigger page are we on
      for (byte i = 0; i < 4; i++)
      {
        if (voiceActive[i])
          LEDrow[4] = bitSet(LEDrow[4], (4 - i)); // indicate active voices
      }

      if (seqRunning)
        LEDrow[currentStep / 8] = LEDrow[currentStep / 8] ^ 1 << (7 - (currentStep % 8)); // toggle the LED for the current step

      if (nowPlaying != 255 && nowPlaying != confirm) // blink nowPlaying
      {
        static boolean blinkOff;
        if ((roughMillis - slowBlink) > 200)
        {
          blinkOff = !blinkOff;
          slowBlink = roughMillis;
        }
        if (blinkOff)
          bitClear(LEDrow[(nowPlaying - (trigPage * 32))/ 8], (7 - ((nowPlaying - (trigPage * 32)) % 8)));
      }

      if (confirm != 255) // fast blink confirm
      {
        static boolean blinkOff;
        if ((roughMillis - fastBlink) > 100)
        {
          blinkOff = !blinkOff;
          fastBlink = roughMillis;
        }
        if (blinkOff)
          bitClear(LEDrow[(confirm - (trigPage * 32))/ 8], (7 - ((confirm - (trigPage * 32)) % 8)));
      }

      if (cued != 255) // fast blink cued
      {
        static boolean blinkOff;
        if ((roughMillis - fastBlink) > 100)
        {
          blinkOff = !blinkOff;
          fastBlink = roughMillis;
        }
        if (blinkOff)
          bitClear(LEDrow[(cued - (trigPage * 32))/ 8], (7 - ((cued - (trigPage * 32)) % 8)));
      }

    }
    break;

  case 3: // live mode
    for (byte i = 0; i < 40; i++)
    {
      if (pressed[i])
      {
        LEDrow[i / 8] = bitSet(LEDrow[i / 8], 7 - (i % 8));
      }
    }
    break;
  }
}

void showNumber()
{
  static byte nums[25] = 
  { 
    B01110010,
    B01010010,
    B01010010,
    B01010010,
    B01110010,

    B01110111,
    B00010001,
    B01110111,
    B01000001,
    B01110111,

    B01010111,
    B01010100,
    B01110111,
    B00010001,
    B00010111,

    B01000111,
    B01000001,
    B01110001,
    B01010001,
    B01110001,

    B01110111,
    B01010101,
    B01110111,
    B01010001,
    B01110001     
  };

  byte offset = 0; // horizontal offset for even numbers 
  byte row = 0; // vertical offset for the start of the digit

  clearLEDs();

  if (number > 199) // add two dots for numbers 200 and over
  {
    LEDrow[3] = B10000000;
    LEDrow[4] = B10000000;
  }

  else if (number > 99) // add a dot for numbers 100 and over
    LEDrow[4] = B10000000;

  //digits under 10
  if ((number % 10) % 2 == 0) // even numbers
    offset = 4;
  else
    offset = 0;
  row = ((number % 10)/2)*5;      

  for (byte j = 0; j < 5; j++)
  {
    for (byte i = 0; i < 3; i++) // 3 pixels across
    {      
      if (bitRead(nums[row + j], offset + i)) // ie. if it returns 1  
        LEDrow[j] = bitSet(LEDrow[j], i);
    }
  }

  // digits over 10
  if (((number / 10) % 10) % 2 == 0) // even numbers
    offset = 4;
  else
    offset = 0;
  row = (((number / 10) % 10)/2)*5;      

  for (byte j = 0; j < 5; j++)
  {
    for (byte i = 0; i < 3; i++) // 3 pixels across
    {      
      if (bitRead(nums[row + j], offset + i)) // ie. if it returns 1  
        LEDrow[j] = bitSet(LEDrow[j], i + 4);
    }
  }
}



















